/* No title for MacOS */
var tTitle = global.DA.isMacOS ? '' : 'Kopano DeskApp';

/* Different icons per OS */
var tIcon = 'app/resources/images/logo.png';
var tIconUnread = 'app/resources/images/logo_unread.png';
if (global.DA.isMacOS) {
	tIcon = 'app/resources/images/tray_icon_mac.png';
	tIconUnread = 'app/resources/images/tray_icon_mac_unread.png';
}
if (global.DA.isLinux) {
	tIcon = 'app/resources/images/tray_icon_linux.png';
	tIconUnread = 'app/resources/images/tray_icon_linux_unread.png';
}
if (global.DA.isWin10OrHigher()) {
	tIcon = 'app/resources/images/tray_icon_win10.png';
	tIconUnread = 'app/resources/images/tray_icon_win10_unread.png';
}

/* Modify the tray icon for KDE */
if (global.DA.isKDE) {
	tIcon = 'app/resources/images/tray_icon_kde.png';
	tIconUnread = 'app/resources/images/tray_icon_kde_unread.png';
}

/**
 * Reload WebApp
 */
function reloadWebApp() {
	if (!global.DA.webAppWindow.nwWindow) {
		return;
	}
	global.DA.webAppWindow.hide();
	global.DA.winWelcome.window.showLoaderForm();
	global.DA.winWelcome.show();
	global.DA.webAppWindow.reload();
}

/**
 * Update the tray icon
 * @param {Boolean} unread True if unread items are available 
 */
function updateTrayIcon(unread) {
	var icon = unread ? tIconUnread : tIcon
	tray.icon = icon;
}

/* Give it a menu */
var menu = new nw.Menu();
var sep = new nw.MenuItem({
	type: 'separator'
});

/* Add menu item to show the webapp window */
var trayOpen = new nw.MenuItem({
	type: 'normal',
	label: gt.gettext('Open DeskApp'),
	click: function() {
		if (global.DA.webAppWindow.nwWindow) {
			global.DA.webAppWindow.show(true);
		}
	}
});
menu.append(trayOpen);

/* Add menu item to show the DeskApp version */
var trayShowVersion = new nw.MenuItem({
	type: 'normal',
	label: 'DeskApp ' + chrome.runtime.getManifest().version,
	enabled: false
});
menu.append(trayShowVersion);
menu.append(sep);

/* Add menu item to clear the browser cache */
var trayClearCache = new nw.MenuItem({
	type: 'normal',
	label: gt.gettext('Clear cache'),
	click: function() {
		nw.App.clearCache();
		reloadWebApp();
	}
});
menu.append(trayClearCache);

/* Add menu item to relaod the WebApp Window */
var trayReload = new nw.MenuItem({
	type: 'normal',
	label: gt.gettext('Reload DeskApp'),
	click: reloadWebApp
});
menu.append(trayReload);

/* Add menu item to close DeskApp */
var trayClose = new nw.MenuItem({
	type: 'normal',
	label: gt.gettext('Close DeskApp'),
	click: function() {
		nw.App.quit();
	}
});
menu.append(trayClose);

/* Add menu item to open DownloadManager */
var downloads = global.DA.requireCustomModule('downloads');
var trayDownloadHistory = new nw.MenuItem({
	type: 'normal',
	label: gt.gettext('Download history'),
	click: function() {
		downloads.openHistory();
	}
});
menu.append(trayDownloadHistory);

/* Add menu item to open the settings window */
var traySettings = new nw.MenuItem({
	type: 'normal',
	label: gt.gettext('Settings\u2026'),
	click: global.DA.openSettings
});
menu.append(traySettings);

window.onbeforeunload = function() {
	if (tray) {
		tray.remove();
		tray = null;
	}
};

/* Create a tray icon */
var tray = new nw.Tray({
	title: tTitle,
	tooltip: 'Kopano DeskApp',
	icon: tIcon,
	/* Only effective on OSX */
	iconsAreTemplates: false
});

tray.menu = menu;
