global.DA.isFirstWebAppLogin = true;

/* DOM elements */
var right = document.getElementById("right");
var btMinimize = document.getElementById("btMinimize");
var btClose = document.getElementById("btClose");

/* NW Window objects*/
global.DA.winWelcome = nw.Window.get();
global.DA.winWelcome.setResizable(false); // A workaround for Linux

/* Add eventlisteners for minimize and close buttons */
btMinimize.addEventListener('click', function(event) {
	global.DA.winWelcome.minimize();
});
btClose.addEventListener('click', function(event) {
	if (global.DA.winPreferences) {
		global.DA.winPreferences.close(true);
	}
	if (global.DA.webAppWindow && global.DA.webAppWindow.nwWindow) {
		global.DA.webAppWindow.close(true);
	}
	if (global.settings.general.clearCacheOnClose) {
		nw.App.clearCache();
	}
	global.DA.winWelcome.close(true);
});

/* check if there is an ini file to imnport */
checkForINI();

/* Read settings from local storage, and store it globally */
global.DA.readSettings();

// Check if profiles need to be updated
DAProfiles.updateProfiles();

var gProfile;
var downloads = global.DA.requireCustomModule('downloads');
var ModalDialog = global.DA.requireCustomModule('modalDialogs');

/* No settings yet, show profile form */
var noProf = (Object.keys(global.settings.profiles).length == 0);
if (noProf) {
	showFirstProfileForm();
} else {
	/* Read the profile settings */
	var defaultProfile = global.settings.general.defaultProfile;
	var autoLogin = global.settings.general.autoLogin;
	showWelcomeForm();
	/* If autoLogin is true directly login*/
	if (autoLogin && defaultProfile) {
		signIn(defaultProfile);
	}
}

chrome.webRequest.onBeforeRequest.addListener(
	function(d) { return {cancel: true}; },
	{ urls: ['*://*/*kopano.php?service=fingerprint'] },
	["blocking"]
);

/**
 * Eventhandler function for the sign in button
 * @param {Event} event event calling the function
 */
function onSignIn(event) {
	event.preventDefault();
	var profileId = document.getElementById('profileSelector').value;
	if (!profileId) {
		return;
	}
	if (document.getElementById('cbDefault').checked) {
		global.DA.setAsDefault(profileId);
	}
	signIn(profileId);
}

/**
 * Executes signing in, asks for password and username if not saved within the profile
 */
function signIn(profileId) {
	gProfile = global.settings.profiles[profileId];
	let isSSO = gProfile.authType === 'sso';
	if (!isSSO && gProfile.requestOptions.form.password === '' && gProfile.requestOptions.form.username === '') {
		var modalCred = new ModalDialog({
				'type': 'credentialsDialog'
			}, document.body, window,
			function() {
				var credentials = JSON.parse(modalCred.getReturnValue());
				gProfile.requestOptions.form.username = credentials.username;
				gProfile.requestOptions.form.password = DAEncryption.encrypt(credentials.password);
				saveProfile(gProfile);
				showLoaderForm();
				createWebAppWindow(gProfile.id);
			});
	} else if (!isSSO && (!gProfile.rememberPassword || gProfile.requestOptions.form.password === '')) {
		var modalPW = new ModalDialog({
			'type': 'passwordDialog'
		}, document.body, window, function() {
			gProfile.requestOptions.form.password = DAEncryption.encrypt(modalPW.getReturnValue());
			if (gProfile.rememberPassword) {
				saveProfile(gProfile);
			}
			showLoaderForm();
			createWebAppWindow(gProfile.id);
		});
	} else {
		showLoaderForm();
		createWebAppWindow(gProfile.id);
	}
}

/**
 * Create a new webapp window and login
 * @param  {String} profileID The ID of the profile that should be used to login
 */
function createWebAppWindow(profileID) {
	allowAutomaticDownloads(gProfile.requestOptions.url);
	global.DA.webAppWindow = null;
	global.DA.webAppWindow = new WebAppWindow(profileID, cbWebAppWindowSuccess, cbWebAppWindowFailure);
	global.DA.webAppWindow.createWindowAndLogin();
}

/**
 * Success callback for the webapp window
 * Shows some info about the login in the console 
 * @param  {object} details details for the webapp login
 */
function cbWebAppWindowSuccess(details) {
	/* Check if DeskApp is the default mail client on Windows */
	global.DA.requireCustomModule('windows').checkIsDefaultMailApp();
}

/**
 * Success callback for the webapp window
 * Shows the error or asks for the password if MAPI_E_LOGON_FAILED occured
 * @param  {object} details details for the webapp login error
 */
function cbWebAppWindowFailure(details) {
	if (details.MAPI_E === "MAPI_E_LOGON_FAILED") {
		var modalPW = new ModalDialog(
			{'type': 'passwordInvalidDialog'}, document.body, window,
			function(){
				gProfile.requestOptions.form.password = DAEncryption.encrypt(modalPW.getReturnValue());
				if (gProfile.rememberPassword) {
					saveProfile(gProfile);
				}
				createWebAppWindow(gProfile, cbWebAppWindowSuccess, cbWebAppWindowFailure);
			},
			function(){
				showWelcomeForm();
			}
		);
	} else {
		showErrorMessage(details.message);
	}
}

/**
 * Shows the error message from cbWebAppWindowFailure
 * @param  {String} msg the error message shown to the user
 */
function showErrorMessage(msg) {
	var error = document.getElementById('error');
	error.innerHTML = msg;
}

/**
 * Open the settings window and create a new profile
 * @param {Event} event event calling the function
 */
function createProfile(event) {
	if (event) {
		event.preventDefault();
	}
	global.DA.openSettingsWindow("", true);
}

/**
 * Show the welcome form
 * @param {Event} event event calling the function
 */
function showWelcomeForm(event) {
	if (event) {
		event.preventDefault();
	}
	var profiles = global.settings.profiles;
	var selectOptions = {};
	for (var profile in profiles) {
		selectOptions[profiles[profile].id] = profiles[profile].name;
	}

	var formConfig = {
		ID: 'welcome',
		header: {
			'element': 'h1',
			'attributes': {
				'style': 'margin-top: 30px',
			},
			'innerHTML': gt.gettext('Welcome'),
		},
		profileSelector: {
			'element': 'select',
			'attributes': {
				'id': 'profileSelector',
			},
			'options': selectOptions,
		},
		newProfileBtn: {
			'element': 'button',
			'attributes': {
				'name': 'buttonNewProfile',
				'id': 'buttonNewProfile',
				'class': 'buttonGrey',
			},
			'innerHTML': gt.gettext('New Profile'),
			'listeners': {
				'click': createProfile,
			}
		},
		cbDefault: {
			'element': 'input',
			'attributes': {
				'type': 'checkbox',
				'name': 'cbDefault',
				'id': 'cbDefault',
				'style': 'margin-top: 10px;'
			},
			'label': gt.gettext('Set as default profile'),
		},
		error: {
			'element': 'div',
			'attributes': {
				'id': 'error'
			}
		},
		signIn: {
			'element': 'input',
			'attributes': {
				'type': 'submit',
				'name': 'submitSignIn',
				'id': 'submitSignIn',
				'class': 'buttonBlue',
				'style': 'position: absolute; left: 20px; bottom: 24px;',
				'value': gt.gettext('Sign in')
			}
		},
		settingsBtn: {
			'element': 'button',
			'attributes': {
				'name': 'settingsButton',
				'id': 'settingsButton',
				'class': 'settingsButton',
				'style': 'position: absolute; right: 24px; bottom: 24px;'
			},
			'innerHTML': gt.gettext('Settings'),
			'listeners': {
				'click': global.DA.openSettings,
			}
		},
	};

	global.DA.addForm(right, formConfig, onSignIn);
	if (global.settings.general.defaultProfile) {
		document.getElementById('profileSelector').value = global.settings.general.defaultProfile;
	}

	document.getElementById('submitSignIn').focus();
}

/**
 * Show the loader form when the webapp window is being loaded
 */
function showLoaderForm() {
	var formConfig = {
		ID: 'loader',
		loadingDiv: {
			'element': 'div',
			'attributes': {
				'class': 'loading',
			}
		},
		error: {
			'element': 'div',
			'attributes': {
				'id': 'error'
			}
		},
		settingsBtn: {
			'element': 'button',
			'attributes': {
				'name': 'settingsButton',
				'id': 'settingsButton',
				'class': 'settingsButton',
				'style': 'margin-top: 30px; margin-left: 20px; margin-right: 4px;'
			},
			'innerHTML': gt.gettext('Settings'),
			'listeners': {
				'click': global.DA.openSettings,
			}
		},
		abortSignIn: {
			'element': 'button',
			'attributes': {
				'name': 'abortButton',
				'id': 'abortButton',
				'class': 'buttonGrey',
				'style': 'float: right; margin-top: 30px;'
			},
			'innerHTML': gt.gettext('Abort login'),
			'listeners': {
				'click': function(event) {
					event.preventDefault();
					global.DA.webAppWindow.cancel();
					showWelcomeForm();
				}
			}
		},
	};
	global.DA.addForm(right, formConfig, signIn);
}

/**
 * Show the first profile form
 * @param {Event} event event calling the function
 */
function showFirstProfileForm(event) {
	if (event) {
		event.preventDefault();
	}

	var formConfig = {
		ID: 'firstProfileForm',
		header: {
			'element': 'h1',
			'innerHTML': gt.gettext('Welcome'),
		},
		createProfileBtn: {
			'element': 'button',
			'attributes': {
				'name': 'createProfileBtn',
				'id': 'createProfileBtn',
				'class': 'buttonGrey',
			},
			'innerHTML': gt.gettext('Create Profile'),
			'listeners': {
				'click': createProfile,
			}
		},
	};

	global.DA.addForm(right, formConfig, createProfile);
	document.getElementById('createProfileBtn').focus();
}

/**
 * Save a single profile
 * @param {Object} profile profile to save
 */
function saveProfile(profile) {
	var tmpSettings = {
		'profiles': {}
	};
	tmpSettings.profiles[gProfile.id] = gProfile;
	global.DA.saveSettings(tmpSettings);
}

/**
 * Allow automatic downloads for a given url
 * @param {String} url allow automatic downloads for url
 */
function allowAutomaticDownloads(url) {
	var parser = document.createElement('a');
	parser.href = url;
	var pattern = parser.protocol + '//' + parser.hostname + '/*';
	chrome.contentSettings.automaticDownloads.set(
		{'primaryPattern': pattern, 'setting': 'allow'}, function(){});
}
